#include "altlib.h"
/* This file contains simplified versions of library
 * routines from the JCMLIB library, sufficient to
 * support the simple test.c program and the portable unit routines
 */
long cs_begs( const char* ibuf );
void cs_numberclean( char* tbuf, long sig, char* buf, long siz );
static void cs_rfclean( char* tbuf );


void fio_dashline( long n )
{
 printf( "----------------------\n" );
}

logical ut_dbg_level( const long i )
{
 return 0;
}

void cs_copy( const char* src, char* dest )
{
 strcpy( dest, src );
}

void cs_upper( char* buf )
{
 if ( !buf ) return;
 while( *buf != '\0' ) 
 {
  *buf = toupper( *buf );
  buf++;
 }
}

logical cs_is_blank( const char* ibuf )
{
 char* buf = (char*)ibuf;
 if ( !buf ) return 1;
 while( *buf == ' ' ) buf++;
 return( *buf  == '\0' );
}

double tpow_d( const long m )
{
 long n = m;
 double result;
 result = 1.0;
 if ( n == 0 ) {
  return( result );
 } else if ( n < 0 ) {
  while( n++ ) result *= 10.0;
  return( 1.0/result );
 } else {
  while( n-- ) result *= 10.0;
  return( result );
 }
}


void cs_get_cx( char** buf, char* token, const long siz, char* seps );

void cs_get_c( char** buf, char* token, const long siz )
{
 cs_get_cx( buf, token, siz, " \t,;" );
}

void cs_get_cs( char** buf, char* token, const long siz )
{
 cs_get_cx( buf, token, siz, ";" );
}

void cs_get_cx( char** buf, char* token, const long sizp, char* seps )
{
 long siz = sizp;
 char* ptr = *buf;
 char* optr = token;
 char c;
 while (*ptr == ' ' || *ptr == '\t' ) ptr++;
 c = *ptr;
 while ( c && !strchr( seps, c ) && siz )
 {
  *optr= c;
  ptr++; optr++; siz--;
  c = *ptr;
 }
 *optr = '\0';
 while (*ptr == ' ' || *ptr == '\t' ) ptr++;
 *buf = ptr;
 if ( *ptr == '\0' ) *buf = NULL;
}

logical cs_eq( const char* ibuf1, const char* ibuf2 )
{
 char* buf1 = (char*)ibuf1;
 char* buf2 = (char*)ibuf2;
 if ( !buf1 )
  return ( buf2 == NULL );
 
 while( *buf1 == *buf2 && *buf1 != '\0' )  {
  buf1++;
  buf2++;
 }
 if ( *buf1 == '\0' ) return( cs_is_blank( buf2 ) );
 if ( *buf2 == '\0' ) return( cs_is_blank( buf1 ) );
 return( UT_FALSE );
}


void fio_tmsg( char* buf )
{
 printf( "%s\n", buf );
}

int cs_index( const char* buf, const char* sub )
{
 char* ptr;

 ptr = strstr( buf, sub );
 if ( ptr == NULL )
  return(0);
 else 
  return( ptr - buf + 1 );
}

long cs_ends( const char* ibuf )
{
 char* buf = (char*)ibuf;
 long size;

 size = strlen( buf );
 buf += size - 1 ;
 while ( size > 0 && *buf-- == ' '  )  size-- ;

 return( size );
}

void cs_copy_siz( const char* isource, char* dest, const long maxlen )
{
 char* source = (char*)isource;
 long siz = maxlen;
 if ( !dest ) return;
 if ( !siz )
 {
  cs_copy( source, dest );
 } else {
  if ( source == NULL ) { *dest= '\0'; return; }
  while ( *source != '\0' && siz )
  {
   *dest = *source;
   source++;
   dest++; 
   siz--;
  }
  *dest = '\0';
 }
}

void cs_detab( char* buf )
{
 while( *buf != '\0' ) {
  if ( *buf == '\t' ) *buf = ' ';
  buf++;
 }
}

char** ar_realloc_c( char** word, const long size, const long n )
{
 char* buf = word[0];
 long i;
 buf = (char*)realloc( buf, n * (size+1) * UT_SZ_C );
 word = (char**)realloc( word, n * UT_SZ_P );
 word[0] = buf;
 for ( i = 1; i < n; i++ )
  word[i] = word[i-1] + size + 1;
 return word;
}

char** ar_alloc_c( const long size, const long n )
{
 char** word;
 char* buf;
 long i;

 /* Reserve n+1 bytes for each char* */
 if ( n == 0 ) return NULL;
 buf  = (char*)calloc(  n * (size + 1),  UT_SZ_C);
 word = (char**) malloc( (size_t) ( n * UT_SZ_P ) );
 word[0] = buf;
 /* Now assign the pointers to each char* */
 for ( i = 1; i < n; i++ ) {
  word[i] = word[i-1] + size + 1;
 }
 return( word );
}

/*
 *--------------------------------------------------------------------------------
 * Free Type FC array with fixed size char*s
 */
void ar_free_c( char** word )
{
 if ( word )
 {
 free( word[0] );
 free( word    );
 }
}




double cs_read_d( char* a )
{
 return (atof(a));
}


long cs_index_char( const char* buf, const char c )
{
 char* ptr;
 ptr = strchr( buf, c );
 if ( ptr == NULL ) {
  return( 0 );
 } else {
  return(  ptr - buf + 1 );
 }
}

logical cs_digit_char(   const char c )
{
 return( isdigit( c ) );
}

long floor_d( const double x );

void frac_d( const double value, long* int_val, double* fraction )
{
 long long_part;

 long_part = floor_d( value );
 *fraction = value - long_part;
 if ( *fraction > 1.0 - 2 * DBL_EPSILON ) { *fraction = 0.0; long_part++; }
 *int_val = long_part;
}


long floor_d( const double x )
{
 long i;
 i = (long)(x+ 2 * DBL_EPSILON);
 if ( x < (double)i ) i--;
 return( i );
}

void cs_put_token( const char* ibuf, char** dest, const char sep, long* siz );

void cs_put_token( const char* ibuf, char** dest, const char sep, long* siz )
{
 char* buf = (char*)ibuf;
 char* ptr = *dest;
 while ( *siz && *buf != '\0' ) 
 {
  *ptr = *buf++; ptr++; (*siz)--;
 }
 if ( *siz && sep )
 {
  *ptr = sep; ptr++;
 }
 *ptr = '\0';
 *dest = ptr;
}

void cs_put_d( const double token, char** buf, long* siz )
{
 TextWord word;
 cs_put_token( cs_write_d(token, word, WORD_SIZE), buf, ' ', siz );
}


void cs_put_c( const char* token, char** buf, long* siz )
{
 cs_put_token( token, buf, ' ', siz );
}

void cs_put_cn( const char* token, char** buf, long* siz )
{
 cs_put_token( token, buf, 0, siz );
}

char* cs_writef_clean_d( const double val, const char* fmt, const logical clean, const long sig, char* buf, const long siz );

char* cs_write_d( const double val, char* buf, const long siz )
{
 return cs_writef_clean_d( val, NULL, UT_TRUE, 0, buf, siz );
}

char* cs_writef_clean_d( const double val, const char* fmt, const logical clean, const long sig, char* buf, const long siz )
{
 TextCard tbuf = " ";
 long i;
 if ( isnan( val ) ) {
  cs_copy_siz( "  -  ", buf, siz );
 } else if ( clean && val == 0.0 ) {
  cs_copy_siz( "0", buf, siz );
 } else {
  i = (long)val;
  if ( clean && (double)i == val )
   sprintf( tbuf, "%ld", i );
  else if ( fmt && !cs_is_blank( fmt ) )
   sprintf( tbuf, fmt, val );
  else if ( fabs( val ) > 1.0e5|| fabs( val ) < 1.0e-3 ) 
   sprintf( tbuf, "%12.5E", val );
  else 
   sprintf( tbuf, "%12.5f", val );
  if ( clean )
   cs_numberclean( tbuf, sig, buf, siz );
  else
  {
   cs_rfclean( tbuf );
   cs_copy_siz( tbuf, buf, siz );
  }   

 }
 return( buf );

}

void cs_numberclean( char* tbuf, long sig, char* buf, long siz )
{
 long head;
 long tail;
 long expt;
 long decimal_point;

 if ( !siz ) return;
/* Standardize leading zero before decimal point */
 cs_rfclean( tbuf );
/* Check where 'E' char is */
 head = cs_begs( tbuf );
/* Skip leading zeroes */
 tbuf += head-1;
 expt = 0;
 if ( tbuf[0] == '-' || tbuf[0] == '+' ) expt++;
 while( isdigit( tbuf[expt] ) || tbuf[expt] == '.' ) expt++;
/* Tail is last digit, 1-based */
 tail = expt;
 if ( tail < 1 )
  return;  /* Error */
/* Locate decimal point, 1-based */
 decimal_point = cs_index_char( tbuf, '.' );

/* Remove trailing zeros in decimal */
 if ( decimal_point > 0 ) {
  while ( tail > decimal_point && tbuf[tail-1] == '0' ) tail--;
  if ( tail == decimal_point ) tail--;
  if ( sig > 0 && tail > decimal_point + sig )
   tail = decimal_point + sig;
 }


 cs_copy_siz( tbuf, buf, tail );
/* Append exponent, unless it is trivial */
 if ( expt > 0 )
 {
  tbuf = tbuf + expt;
  if ( !cs_is_blank( tbuf ) && !cs_eq( tbuf, "E+00" ) ) 
   cs_copy( tbuf, buf+tail );
 }

}

/*
 * Clean a number - standardize leading zero.
 * (1) Change  "-." to "-0."
 * (2) Change  " ." to "0."
 */
static void cs_rfclean( char* tbuf )
{
 long j;
 j = cs_begs( tbuf );
 if ( j > 1 ) {
  if( cs_eq_ss( tbuf, j-1, j+1, " -." ) ) {
   cs_put_ss( "-0.", tbuf, j-1, j+1 );
  } else if ( cs_eq_ss( tbuf, j-1, j, " ." )) {
   cs_put_ss( "0.", tbuf, j-1, j );
  }
 }
}


long cs_begs( const char* ibuf )
{
 char* buf = (char*)ibuf;
 long size;
 long pos;

 size = strlen( buf );
 pos = 1;
 while ( pos < size && *buf == ' ' ) { buf++; pos++; }

 return( pos );
}

void ut_err_util( char* buf, long code )
{
 return;
}

long ar_cmatch_c( const char* buf, char** array, const long n )
{
 long no;

 for ( no = 0; no < n; no++ )
  {
  if ( array[no] && cs_eq( buf, array[no] ) ) return( no );
   }
 return( -1 );
}

long rmod_i( const long value, long* remainder, const long divisor )
{
 long quotient;
 quotient = floor_d( (double)value / (double) divisor );
 *remainder = value - divisor * quotient;
 return( quotient );
}


void cs_put_ss( const char* isource, char* dest, const long pos1, const long pos2 )
{
 char* source = (char*)isource;
 long siz;
 siz = pos2 - pos1 + 1;
 dest += pos1 - 1;
 while ( siz-- ) {
  if ( *source != '\0' ) {
   *dest = *source++;
  } else {
   *dest = ' ';
  }
  dest++;
 }
}

logical cs_eq_siz( const char* ibuf1, const char* ibuf2, const long rsiz )
{
 char* buf1 = (char*)ibuf1;
 char* buf2 = (char*)ibuf2;
 long siz = rsiz;
 char c1;
 char c2;

 if ( !buf1 )
  return ( buf2 == NULL );

 while( siz-- ) {
  c1 = *buf1;
  c2 = *buf2;
  if ( c1 == '\0' )
  {
   if ( c2 == '\0' ) return UT_TRUE;
   if ( c2 != ' ' ) return UT_FALSE;
   buf2++;
  }
  else if ( c2 == '\0' )
  {
   if ( c1 != ' ' ) return UT_FALSE;
   buf1++;
  }
  else
  {
   if ( c1 != c2 ) return UT_FALSE;
   buf1++;
   buf2++;
  }
 }
 return( UT_TRUE );
}

logical cs_eq_ss( const char* buf, const long pos1, const long pos2, const char* comp )
{
 long siz;
 char* ptr;

 ptr = (char*)buf + pos1 - 1;
 siz = pos2 - pos1 + 1;
 return( cs_eq_siz( ptr, comp, siz ) );
}

FioFile fio_open_ar( const char* filename )
{
 FioFile file;

 file = fopen( filename, "r" );
 if (!file) printf( "No file %s\n", filename );
 return file;
}

logical fio_read_line( FioFile file, char* line, long lmax )
{
 char* fst;

 cs_copy( " ", line );
 if ( !file )
  return 0;

 fst = fgets( line, lmax, file );
 if ( !fst ) return 0;
 cs_remove_nl( line );
 return 1;
}

void cs_remove_nl( char* ptr )
{
 long length;

 length = strlen( ptr );
 if ( length ) {
  ptr += length - 1;
  while ( length && ( *ptr == '\n' || *ptr == '\r' ) ) { *ptr-- = '\0'; length--; }
 }
}


logical fio_file_close( FioFile file )
{
 if ( file )
  fclose( file );
 return 1;
}
