#include "unitlib.h"
#include "si.h"


/* Return the SI prefix char* */


void si_getpfx( long exponent, char* pfx, long* remainder )
{
 long si_step;
 *remainder = 0;
 if ( exponent == 2 ) {
  *pfx = ut_constants_siu[s_hecto];
 } else if ( exponent == 1 ) {
  *pfx = ut_constants_siu[s_deka];
 } else if ( exponent == -1 ) {
  *pfx = ut_constants_siu[s_deci];
 } else if ( exponent == -2 ) {
  *pfx = ut_constants_siu[s_centi];
 } else if ( exponent >= -UT_SI_NPFX && exponent <= UT_SI_NPFX ) {  
  si_step = rmod_i( exponent, remainder, 3 );
  *pfx = ut_constants_siu[si_step+UT_SI_NPFX];
 } else {
  *pfx = ' ';
  *remainder = exponent;
 }
}

/*
 *   Return the SI exponent given the prefix.
 *   Also return a status, which is false if the prefix
 *   is not valid.
 */
long si_exp( char* pfx, logical* ok )
{
 long pos;
 long step;
 *ok = UT_TRUE;
 for ( pos = -UT_SI_NPFX; pos <= +UT_SI_NPFX+4; pos++ ) {
  step = pos + UT_SI_NPFX;
  if ( *pfx == ut_constants_siu[step] ) { 
   *ok = UT_TRUE;
   return( ut_constants_siv[step] );
  }
 }

 if ( cs_eq( pfx, UT_SI_MICRO ) ) {
  return( -6 );
 }
 *ok = UT_FALSE;
 return( 0 );
}



void si_check( char* unit, long* factor, long* pos )
{
 long slen;
 char utmp[2];
 logical ok;
 slen = cs_ends( unit );
 if ( slen > 3 && cs_eq_siz( unit, UT_SI_MICRO, 3 ) ) {
  *factor = -6;
  *pos = 4;
 } else if ( slen == 1 ) {
  *factor = 0;
  *pos = 1;
 } else {
  cs_copy_siz( unit, utmp, 1 );
  *factor = si_exp( utmp, &ok );
  *pos = ok ? 2 : 1;
 }
}

